/*
	Khepera3.java
	Example using Khepera3 with Java and the Korebot 2
	
	Copyright (C) 2012 K-TEAM SA
	
	version 
		1.0 20120120 JT: draft
		
*/


class Khepera3
{
	// methods available
	private native int init();
	private native int getRevision();
	private native int getVersion();
	private native int setMotSpeed(int left, int right);
	private native int getMotLeftSpeed();
	private native int getMotRightSpeed();
	private native int setMotPosition(int left, int right);
	private native int getMotLeftPosition();
	private native int getMotRightPosition();
	private native int getIRProximity(int [] data);
	private native int getIRAmbiant(int [] data);
	private native int getUSValues(int [] data);
	private native float getBatteryState(int state);
	


	// class initialisation
	static{ 
		      System.loadLibrary("Khepera3"); // loads libKhepera3.so
	}

	// main program
	public static void main(String[] args)
	{
		
		int us_values[], ir_values[], motor_values[];
		
		Khepera3 kh3 = new Khepera3(); // create robot object
		
		
		us_values = new int[5];
		ir_values = new int[11];
		 
		
		int err=0;
		// initialise the robot  
		if ((err=kh3.init()) !=0)
		{
			System.out.println("Error number " + err + " : could not initialise the Khepera 3!");
			return;
		}
		  
		System.out.println("Revision: " + kh3.getRevision() + " version: " + kh3.getVersion());
		System.out.println("Battery voltage: " + kh3.getBatteryState(0) + "   remaining capacity: " + kh3.getBatteryState(5) + " %");
		
		/* For ever loop */
  while(true)
  {
		/* Wait 2 seconds */
		try{
   		Thread.sleep(2000);
		} catch (InterruptedException e){
   		e.printStackTrace();
		}

		/* Tell to the motor controller to move the Khepera III forward with speed control*/
		int motspeed= (int)Math.round(40.0*218.72); // compute internal speed (factor 218.72 for version 3.0, 144.01 for smaller)
		System.out.println("Moving forward of " + 40*5 + "mm with speed " + 40 + " of mm/s ...");
		
		kh3.setMotSpeed(motspeed,motspeed);
		
		/* Wait 5 seconds */
				try{
   		Thread.sleep(5000);
		} catch (InterruptedException e){
   		e.printStackTrace();
		}

		/* Tell to the motor controller to stop the Khepera III  */
		kh3.setMotSpeed(0,0);
	
		/* Wait 5 seconds */
				try{
   		Thread.sleep(5000);
		} catch (InterruptedException e){
   		e.printStackTrace();
		}	
	
		// move backward of 5 cm with position control

		// get old position
		int lpos=kh3.getMotLeftPosition();
		int rpos=kh3.getMotRightPosition();
	
		int fpos = (int)Math.round(-20.0*10.0/0.03068);  //  (1 pulse unit = 0.03068mm for version 3.0, 0.047 for smaller)
		System.out.println("Moving backward of "+ -200 +" mm in " + fpos  + " pulses...");
		System.out.println(""); 
		kh3.setMotPosition(lpos+fpos,rpos+fpos);
	
		/* Wait 10 seconds */
		try{
   		Thread.sleep(10000);
		} catch (InterruptedException e){
   		e.printStackTrace();
		}
	
		// get ir sensor values
		if(kh3.getIRProximity(ir_values)==0)
		{ 
			System.out.println("Getting IR proximity values (near 0 = obsctacle is near, near 4095 = obstacle is far).");	
			for (int i=0;i<11;i++)      
			{
				System.out.println("IR sensor " + i + " : "+ ir_values[i]);
			}

		}	 
		else
		{	
			System.out.println("Error getting IR values!");		
		}
		System.out.println("");       
	   
				 
		// get and print us sensors
	
		if(kh3.getUSValues(us_values)==0)
		{
			
			System.out.println("US sensors : distance [cm]");
			System.out.println("left 90   : "+ us_values[0]);
			System.out.println("left 45   : "+ us_values[1]);
			System.out.println("front     : "+ us_values[2]);
			System.out.println("right 45  : "+ us_values[3]);
			System.out.println("right 90  : "+ us_values[4]);
			System.out.println("");                                      
		 }
		 else
		{	
			System.out.println("Error getting US values!");		
		}	
		  
	
    
  } // while
		

		
	}
        
}

