/*
	libKhepera3.c library for interfacing C and Java for the Khepera3
	
	Copyright (C) 2012 K-TEAM SA
	
	version
		1.1 20130708 JT: corrected Java_Khepera3_getMotLeftSpeed and Java_Khepera3_getMotRightSpeed names
		1.0 20120120 JT: draft
	
*/

#include "Khepera3.h"
#include <korebot/korebot.h>

static knet_dev_t * dsPic;
static knet_dev_t * mot1;
static knet_dev_t * mot2;


/*******************************************************
 *!
 * Java_Khepera3_init initiliase the libkorebot and the khepera 3
 *	\param *env interface to the JVM
 *	\param obj reference to the method that called this native code
 *  \return 0 => OK, 0 =!  => ERROR
*/
JNIEXPORT jint JNICALL 
 Java_Khepera3_init(JNIEnv *env, jobject obj)
 {
 		char Buffer[16];
 		int err;
 
 		
 		#if (KB_VERSION == 1 && KB_REVISION < 18)
			kh3_init();
 		
 		#else
 		if ((err=kh3_init()) <0)  // error in kh3_init until version 1.17 inclusive , no return value if success!
    {
    	
    	printf("\nERROR: in Java_Khepera3_init kh3_init() returned %d!\n",err);
    	return -1; // error occured
    }
 		#endif
 		

		/* open various socket and store the handle in their respective pointers */
		dsPic  = knet_open( "Khepera3:dsPic" , KNET_BUS_I2C , 0 , NULL );	
		if (dsPic == NULL)
    {
    	return -2; // error occured
    }
		
		mot1  = knet_open( "Khepera3:mot1" , KNET_BUS_I2C , 0 , NULL );
		if (mot1 == NULL)
    {
    	return -3; // error occured
    }
    
		mot2  = knet_open( "Khepera3:mot2" , KNET_BUS_I2C , 0 , NULL );
		if (mot2 == NULL)
    {
    	return -4; // error occured
    }

		/* initialize the motor controler 1 */
		kmot_SetMode( mot1 , kMotModeIdle );
		kmot_SetSampleTime( mot1 , 1550 );
		kmot_SetMargin( mot1 , 6 );
		kmot_SetOptions( mot1 , 0x0 , kMotSWOptWindup | kMotSWOptStopMotorBlk 
			| kMotSWOptDirectionInv );       
		kmot_ResetError( mot1 );
		kmot_SetBlockedTime( mot1 , 10);
		kmot_ConfigurePID( mot1 , kMotRegSpeed , 620 , 3 , 10 );
		kmot_ConfigurePID( mot1 ,kMotRegPos,600,20,30);
		kmot_SetSpeedProfile(mot1 ,15000,30);

		/* initialize the motor controler 2 */
		kmot_SetMode( mot2 , kMotModeIdle );
		kmot_SetSampleTime( mot2 , 1550 );
		kmot_SetMargin( mot2 , 6 );
		kmot_SetOptions( mot2 , 0x0 , kMotSWOptWindup | kMotSWOptStopMotorBlk );       
		kmot_ResetError( mot2 );
		kmot_SetBlockedTime( mot2 , 10);
		kmot_ConfigurePID( mot2 , kMotRegSpeed , 620 , 3 , 10 );
		kmot_ConfigurePID( mot2 ,kMotRegPos,600,20,30);
  	kmot_SetSpeedProfile(mot2 ,15000,30);
    
    
    // configure us sensors
				 
		if((err=kh3_configure_os((char *)Buffer, 0, 31, dsPic))<=0) // all us sensors active 
		{
			printf("\nERROR: in Java_Khepera3_init kh3_configure_os() returned %d!\n",err);
			return -5; // error occured
		}
    
    
    return 0;
    
    
 }
 
 
/*******************************************************
 *!
 * Java_Khepera3_getRevision get revision of the khepera 3 internal software
 *
 *	\param *env interface to the JVM
 *	\param obj reference to the method that called this native code
 *  \return revision
 */
JNIEXPORT jint JNICALL 
 Java_Khepera3_getRevision(JNIEnv *env, jobject obj)
{
  char Buffer[16];
  
  if(kh3_revision((char *)Buffer, dsPic) <=0){
  	return -1;
  }

  return (Buffer[1] | Buffer[2]<<8);
}

/*******************************************************
 *!
 * Java_Khepera3_getVersion get version of the khepera 3 internal software
 *	\param *env interface to the JVM
 *	\param obj reference to the method that called this native code
 *
 *  \return version
 */
JNIEXPORT jint JNICALL 
 Java_Khepera3_getVersion(JNIEnv *env, jobject obj)
{  
  char Buffer[16];
  
  if(kh3_revision((char *)Buffer, dsPic) <=0){
  	return -1;
  }

  return (Buffer[3] | Buffer[4]<<8);
}

/*******************************************************
 *!
 * Java_Khepera3_setMotSpeed set motors speed [encoder units/s]
 *
 *	\param *env interface to the JVM
 *	\param obj reference to the method that called this native code
 *	\param left left speed
 *	\param right right speed
 *  \return 0 
 */
JNIEXPORT jint JNICALL Java_Khepera3_setMotSpeed
  (JNIEnv *env, jobject obj, jint left, jint right)
{    
 	kmot_SetPoint( mot1 , kMotRegSpeed , left );
	kmot_SetPoint( mot2 , kMotRegSpeed , right );
  
  return 0;  
}

/*******************************************************
 *!
 * Java_Khepera3_getMotLeftSpeed get left motor speed [encoder units/s]
 *
 *	\param *env interface to the JVM
 *	\param obj reference to the method that called this native code
 *	\return left speed
 */
JNIEXPORT jint JNICALL Java_Khepera3_getMotLeftSpeed
  (JNIEnv *env, jobject obj)
{     
 	return kmot_GetMeasure(mot1, kMotRegSpeed); 
}

/*******************************************************
 *!
 * Java_Khepera3_getMotLeftRight get right motor speed [encoder units/s]
 *
 *	\param *env interface to the JVM
 *	\param obj reference to the method that called this native code
 *	\return right speed
 */
JNIEXPORT jint JNICALL Java_Khepera3_getMotRightSpeed
  (JNIEnv *env, jobject obj)
{     
 	return kmot_GetMeasure(mot2, kMotRegSpeed);	
}


/*******************************************************
 *!
 * Java_Khepera3_setMotPosition set motors position [encoder units]
 *
 *	\param *env interface to the JVM
 *	\param obj reference to the method that called this native code
 *	\param left left position
 *	\param right right position
 *  \return 0 
 */
JNIEXPORT jint JNICALL Java_Khepera3_setMotPosition
  (JNIEnv *env, jobject obj, jint left, jint right)
{  
	kmot_SetPoint( mot1 , kMotRegPosProfile , left );
	kmot_SetPoint( mot2 , kMotRegPosProfile , right );
  
  return 0;
}

/*******************************************************
 *!
 * Java_Khepera3_getMotLeftPosition get left motor position [encoder units]
 *
 *	\param *env interface to the JVM
 *	\param obj reference to the method that called this native code
 *	\return left position
 */
JNIEXPORT jint JNICALL Java_Khepera3_getMotLeftPosition
  (JNIEnv *env, jobject obj)
{     
 	return kmot_GetMeasure(mot1, kMotRegPos);
}

/*******************************************************
 *!
 * Java_Khepera3_getMotRightPosition get right motor position [encoder units]
 *
 *	\param *env interface to the JVM
 *	\param obj reference to the method that called this native code
 *	\return right position
 */
JNIEXPORT jint JNICALL Java_Khepera3_getMotRightPosition
  (JNIEnv *env, jobject obj)
{     
 	return kmot_GetMeasure(mot2, kMotRegPos); 
}

/*******************************************************
 *!
 * Java_Khepera3_getIRProximity get IR proximity values
 *
 *	\param *env interface to the JVM
 *	\param obj reference to the method that called this native code
 *	\param data array of long containing the data
 *	\return 0 OK, < 0 error
 */

JNIEXPORT jint JNICALL Java_Khepera3_getIRProximity
  (JNIEnv *env, jobject obj, jintArray data)
{
	int i;
	char Buffer[100];

  jboolean isCopy;
	jint *datapt;

		// get ir sensor
	if(kh3_proximity_ir((char *)Buffer, dsPic) < 0)
	{	
		return -1;
	}

	datapt = (*env)->GetIntArrayElements(env, data, &isCopy);	

  for (i=0;i<11;i++)
  {
    datapt[i]=(Buffer[i*2+1] | Buffer[i*2+2]<<8);
  }
  
  (*env)->ReleaseIntArrayElements(env, data, datapt, 0);
   
	return 0;	    
}

/*******************************************************
 *!
 * Java_Khepera3_getIRAmbiant get US values
 *
 *	\param *env interface to the JVM
 *	\param obj reference to the method that called this native code
 *	\param data array of long containing the data
 *	\return 0 OK, < 0 error
 */

JNIEXPORT jint JNICALL Java_Khepera3_getUSValues
  (JNIEnv *env, jobject obj, jintArray data)
{
	int i,err=0;
	char Buffer[100];

  jboolean isCopy;
	jint *datapt;



	datapt = (*env)->GetIntArrayElements(env, data, &isCopy);	

  for (i=0;i<5;i++)
  {
  	if(kh3_measure_us((char *)Buffer, i+1, dsPic)>=0)
		 {
				datapt[i] = (Buffer[0*8+3] | Buffer[0*8+4]<<8);    // get first echo distance                                   
		 }
     else
       err--;
  }
  
  (*env)->ReleaseIntArrayElements(env, data, datapt, 0);
   
	return err;	    
}

/*******************************************************
 *!
 * Java_Khepera3_getBatteryState get battery state
 *
 *	\param *env interface to the JVM
 *	\param obj reference to the method that called this native code
 *	\param state state index to inquire
 *  \return state value;  -1234 or -2345 if error 
 */ 

JNIEXPORT jfloat JNICALL Java_Khepera3_getBatteryState
  (JNIEnv *env, jobject obj, jint state)
{
	int i,err=0;
	char Buffer[100];
	float ret=0;

	// get battery voltage
	 if(kh3_battery_voltage((char *)Buffer, state, dsPic) < 0)
	 {
	 	return -1234;
	 }
	 
	 if (state <0 || state > 5)
	 {
	 	return -2345;
	 }  


	switch(state)
	{
		case 0: // Voltage
		case 1: // current
		case 2: // average current
			ret=(float)(Buffer[1] | Buffer[2]<<8)+(float)(Buffer[3] | Buffer[4]<<8)/10000.0;
			break;
		case 3: // absolute remaining capacity
		case 5: // relative remaining capacity
			ret=(float)(Buffer[1] | Buffer[2]<<8);
			break; // temperature of the battery
		case 4:
			ret=(float)(Buffer[1] | Buffer[2]<<8)+(float)(Buffer[3] | Buffer[4]<<8)/10.0;
			break;	
			
	}

	return ret;	
	 
	}
